"""waila - What am I looking at"""

from inspect import getdoc
from time import ctime
from typing import Any, NoReturn
from datetime import datetime


def waila(obj: Any, **kwargs) -> Any | NoReturn:
  """Prints each entry in obj.__dir__ excluding dunder methods unless
  specified by flag:
  >>> waila(list)
  """
  out = []
  dunder = kwargs.get('dunder', None)
  print_ = kwargs.get('print', True)
  return_ = kwargs.get('getReturn', None)
  save_ = kwargs.get('save', None)
  now = datetime.now()
  timeFid = now.strftime('%F%H%M%S')
  stdFid = 'waila_of_%s%s.txt' % (obj, timeFid)
  fid_ = kwargs.get('fid', None if save_ is None else stdFid)
  if dunder is None:
    dunder = kwargs.get('magic', False)
  dirs = [i for i in dir(obj) if (i[:2] != '__' or dunder)]
  for i in dirs:
    out.append({
      'name': i,
      'type': type(obj.__dict__.get(i, None)),
      'help': getdoc(obj.__dict__.get(i, None)),
    })
  outLines = ['WAILA of object: %s\n' % (obj), ]
  for item in out:
    entry = 'Name: %s \nHelp: %s\n' % (item['name'], item['help'])
    entry += 'END OF ENTRY FOR %s\n' % (item['name'])
    entry += 50 * '*'
    entry += '\n\n'
    outLines.append(entry)
  endMsg = 'Generated by waila - "what am I looking at" on %s.\n'
  endMsg = endMsg % (ctime())
  endMsg += 'waila is a python utility developed by Asger Jon Vistisen'
  outLines.append(endMsg)
  if save_ or fid_:
    with open(fid_, 'w') as f:
      f.writelines(outLines)
  if print_:
    for line in outLines:
      print(line)
  if return_:
    return out
  return None
